﻿using System.Collections.Generic;

using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Media;

namespace TestProject
{
    /// <summary>
    /// 윈도우 헬퍼
    /// </summary>
    public class WindowHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 활성 윈도우 리스트
        /// </summary>
        private static List<Window> _activeWindowList = new List<Window>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 루트 테마 - RootTheme

        /// <summary>
        /// 루트 테마
        /// </summary>
        public static ElementTheme RootTheme
        {
            get
            {
                foreach(Window window in _activeWindowList)
                {
                    if(window.Content is FrameworkElement rootElement)
                    {
                        return rootElement.RequestedTheme;
                    }
                }

                return ElementTheme.Default;
            }
        }

        #endregion
        #region 활성 윈도우 리스트 - ActiveWindowList

        /// <summary>
        /// 활성 윈도우 리스트
        /// </summary>
        public static List<Window> ActiveWindowList
        {
            get
            {
                return _activeWindowList;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 윈도우 추적하기 - TrackWindow(window)

        /// <summary>
        /// 윈도우 추적하기
        /// </summary>
        /// <param name="window">윈도우</param>
        public static void TrackWindow(Window window)
        {
            window.Closed += (sender, e) =>
            {
                _activeWindowList.Remove(window);
            };

            _activeWindowList.Add(window);
        }

        #endregion
        #region 윈도우 생성하기 - CreateWindow()

        /// <summary>
        /// 윈도우 생성하기
        /// </summary>
        /// <returns>윈도우</returns>
        public static Window CreateWindow()
        {
            Window window = new Window
            {
                SystemBackdrop = new MicaBackdrop()
            };

            TrackWindow(window);

            return window;
        }

        #endregion
        #region 엘리먼트를 위한 윈도우 구하기 - GetWindowForElement(element)

        /// <summary>
        /// 엘리먼트를 위한 윈도우 구하기
        /// </summary>
        /// <param name="element">엘리먼트</param>
        /// <returns>윈도우</returns>
        public static Window GetWindowForElement(UIElement element)
        {
            if(element.XamlRoot != null)
            {
                foreach(Window window in _activeWindowList)
                {
                    if(element.XamlRoot == window.Content.XamlRoot)
                    {
                        return window;
                    }
                }
            }

            return null;
        }

        #endregion
    }
}